// CONFIG.CPP
// Processes MAILER.CONF, the configuration file for mailer.
// 10/25/2000 -

// LICENSE
// Copyright (c) 2002-2004, Carson F. Ball
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following
// conditions are met:
//
// o Redistributions of source code must retain the above copyright
//   notice, this list of conditions, and the following disclaimer.
//
// o Redistributions in binary form must reproduce the above
//   copyright notice, this list of conditions and the following
//   disclaimer in the documentation and/or other materials
//   provided with the distribution.
//
// o Neither the name of the author or of other contributors may be
//   used to endorse or promote products derived from this software
//   without prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.

// Define this as being the configuration routines.
#define  CONFIG

#include <stdio.h>   // Needed for fopen.
#include <stdlib.h>  // Needed for atoi.
#include <string.h>  // Needed for strstr.

#include "cgi.h"
#include "config.h"
#include "misc.h"

// +-------------------------------------------------------------------------+
// |FUNCTION:  GetConf                                                       |
// +-------------------------------------------------------------------------+
// |INPUT:     Info     - The structure to hold the information in.          |
// |           CurPath  - The path that Mailer is running in.                |
// |OUTPUT:    Info     - The filled structure.                              |
// |RETURN:    Error code (0 on no error).                                   |
// +-------------------------------------------------------------------------+
// |FILE*      Config   - The valid fields for the configuration file.       |
// |SCHAR**    VALUES   - The file buffer for the configuration file.        |
// |char*      Field    - The current field from the configuration file.     |
// |char*      Path     - The path of Mailer's configuration file.           |
// |char**     Line     - A line of input from the configuration file.       |
// |INT8       Count    - A counter for loops.                               |
// |INT8       Pos      - The position within a string.                      |
// |INT8       Set      - The number of currently set entries.               |
// |INT8       EFlag    - A flag to store error values in.                   |
// +-------------------------------------------------------------------------+
// |This function gets the information from the configuration file.          |
// +-------------------------------------------------------------------------+
INT32 GetConf (CONF* Info, const char* CurPath) {
   SCHAR* Values [NFIELDS] = {"IP:", "PORT:", "VALID:", "DOMAIN:"};
   char*  Field;
   char   Line   [80];

   Info->NumValid = 0;
   INT8 EFlag     = 0;

   char* Path     = new char [strlen (CurPath) + strlen (CONFFILE) + 2];

   strcpy (Path, CurPath);
   strcat (Path, "/");
   strcat (Path, CONFFILE);

   // Open the configuration file for reading.
   FILE* Config = fopen (Path, "rt");

   delete [] Path;

   if (!Config) {
      return (1);
   }

   // Get lines until EOF is encountered.
   while (GetLine (Config, Line)) {
      // Ignore comment lines.
      if (*Line != '#') {
         // Parse the information in the non-comment lines.
         Field = Parse (Line, Values);

         // Check to see if this line contains an IP address.
         if (!strcmpi (Field, *Values)) {
            EFlag          |= IPA;
            Info->IP        = new char [strlen (Line) + 1];
            strcpy (Info->IP, Line);
         }
         // Check to see if this line contains a port number.
         else if (!strcmpi (Field, Values [1])) {
            EFlag          |= PORT;
            Info->Port      = atoi (Line);
         }
         // Check to see if this line contains a valid server addresses.
         else if (!strcmpi (Field, Values [2])) {
            EFlag          |= VALID;
            INT8 Set        = Info->NumValid;
            Info->NumValid += NumAddrs (Line);

            // Is this is the first VALID line (has memory be set yet)?
            if (!Set) {
               // Allocate the memory for the valid values.
               Info->Valid  = (char**) malloc (RESSIZE);
            }
            else {
               // Allocate more memory for the valid values.
               Info->Valid = (char**) realloc (Info->Valid, RESSIZE);
            }

            INT8 Pos       = 0;
            INT8 Count     = Set;

            // Read in all of the valid values.
            do {
               // Allocate enough room for an 80 character server address.
               Info->Valid [Count] = (char*) malloc (80);

               sscanf  (&Line [Pos], "%s", Info->Valid [Count]);
               Pos += strlen (Info->Valid [Count]) + 1;

               // Storing the return value is not necessary since the new
               // size is smaller.  The start address should remain the same.
               realloc (Info->Valid [Count], strlen (Info->Valid [Count]));
            } while (++Count < Info->NumValid);
         }
	 else if (!strcmpi (Field, Values [3])) {
	    EFlag          |= DOMAIN;
	    Info->Domain    = new char [strlen (Line) + 1];
	    strcpy (Info->Domain, Line);
	 }
      }
   }

   // Close the configuration file.
   fclose (Config);

   // Return any error that was found.
   if (!(EFlag & PORT)) {
      return (2);
   }
   if (!(EFlag & IPA)) {
      return (3);
   }
   if (!(EFlag & VALID)) {
      return (4);
   }
   if (!(EFlag & DOMAIN)) {
     return (15);
  }

   return (0);
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  GetLine                                                       |
// +-------------------------------------------------------------------------+
// |INPUT:     FBuff    - The buffer of the input file.                      |
// |           String   - The memory address for the string to return.       |
// |OUTPUT:    String   - The string read from the input file                |
// |RETURN:    The number of bytes read from the file.  On EOF, this value   |
// |           will be 0.                                                    |
// +-------------------------------------------------------------------------+
// |int        InByte   - The byte read from the file.                       |
// |INT8       Flag     - A flag to determine whether the line has ended.    |
// |INT8       Pos      - The current position in the string.                |
// +-------------------------------------------------------------------------+
// |This function gets a complete line of text from a file buffer.           |
// +-------------------------------------------------------------------------+
INT8  GetLine (FILE* FBuff, char* String) {
   int    InByte;
   
   INT8 Pos  = 0;         // Set the position to the beginning of the string.
   INT8 Flag = 0;         // Clear the flag.

   do {
      if ((InByte = fgetc (FBuff)) == EOF) {
         Flag = 1;
      }
      else if (((INT8) InByte == 10) || ((INT8) InByte == 13)) {
         if (Pos) {
            Flag = 2;
         }
      }
      else {
         String [Pos] = (INT8) InByte;
         ++Pos;
      }
   } while (!Flag);

   String [Pos] = 0;

   // Return the number of characters in the line.
   // If the end of the file has been reached, and no characters have been
   // found (Pos == 0), NULL will be returned.
   return (Pos);
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  Parse                                                         |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The line to parse for field names.                 |
// |OUTPUT:    String   - The string containing the field value.             |
// |RETURN:    The string containing the field name.                         |
// +-------------------------------------------------------------------------+
// |char*      Pos      - The position within the string.                    |
// |INT8       Loop     - A variable used in loops.                          |
// |INT8       Len      - The length of the field name.                      |
// +-------------------------------------------------------------------------+
// |This function searches a line of text for one of several field names.    |
// +-------------------------------------------------------------------------+
char* Parse (char* String, char* const* Values) {
   char*  Pos;

   INT8 Loop = 0;

   // Search the line until a field is found or no more fields exist.
   do {
      Pos = strstr (strupr (String), Values [Loop]);
      if (!Pos) {
         ++Loop;
      }
   } while ((Loop < NFIELDS) && !Pos);

   if (Pos) {
      Pos = Values [Loop];

      INT8 Len = strlen (Values [Loop]);
      if (String [Len] == ' ') {
         ++Len;
      }

      memmove (String, &String [Len], strlen (String) - Len);
      String [strlen (String) - Len] = 0;
   }

   return (Pos);
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  NumAddrs                                                      |
// +-------------------------------------------------------------------------+
// |INPUT:     Addrs    - The list of valid server addresses.                |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    The number of valid server addresses.                         |
// +-------------------------------------------------------------------------+
// |char*      Pos      - The position within the string.                    |
// |INT32      Num      - The number of server addresses.                    |
// +-------------------------------------------------------------------------+
// |This function gets the number of server addresses listed in the          |
// |configuration file.                                                      |
// +-------------------------------------------------------------------------+
INT32 NumAddrs (char* Addrs) {
   // If there are no addresses, exit the function.
   if (!strlen (Addrs)) {
      return (0);
   }

   INT32 Num = 1;     // Initialize the number of valid server addresses to 1.
   char* Pos = Addrs; // Set the position to the beginning of the string.

   // Continue while the end of the string has not been reached.
   do {
      // If a space was encountered, increment the number of addresses.
      if (*Pos == ' ') {
         ++Num;
      }
   } while (++Pos < Addrs + strlen (Addrs));

   return (Num);
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  Server                                                        |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    The name of the calling server (NULL if error).               |
// +-------------------------------------------------------------------------+
// |char*      ServName - The name of the calling server.                    |
// +-------------------------------------------------------------------------+
// |This function gets the name of the calling server, removes the page      |
// |name, and returns this string or NULL if an error ocurred.               |
// +-------------------------------------------------------------------------+
char* Server (void) {
   char* ServName = getenv ("HTTP_REFERER");
   if (ServName) {

      // The referring server should always begin with http://.
      // And, the referring server should always end with /.
      ServName += 7;
      ServName [strlen (ServName) - strlen (strchr (ServName, '/'))] = '\0';
   }

   // Will return NULL if Mailer was not called from a server.
   return (ServName);
}
