// MAILER.CPP
// The main body of the mailer program.
// 10/25/2000 -

// LICENSE
// Copyright (c) 2002-2004, Carson F. Ball
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following
// conditions are met:
//
// o Redistributions of source code must retain the above copyright
//   notice, this list of conditions, and the following disclaimer.
//
// o Redistributions in binary form must reproduce the above
//   copyright notice, this list of conditions and the following
//   disclaimer in the documentation and/or other materials
//   provided with the distribution.
//
// o Neither the name of the author or of other contributors may be
//   used to endorse or promote products derived from this software
//   without prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.

// Define this as being the main routines.
#define  MAIN

#include <stdio.h>  // Needed for printf.
#include <stdlib.h> // Needed for exit.
#include <string.h> // Needed for strlen.

#ifdef __linux__
#include <unistd.h>
#endif

#include "sock.h"
#include "cgi.h"
#include "config.h"
#include "errors.h"
#include "mailio.h"
#include "mtypes.h"
#include "misc.h"
#include "version.h"

// +-------------------------------------------------------------------------+
// |FUNCTION:  main                                                          |
// +-------------------------------------------------------------------------+
// |INPUT:     Info     - The structure to hold the information in.          |
// |           CurPath  - The path that Mailer is running in.                |
// |OUTPUT:    Info     - The filled structure.                              |
// |RETURN:    Error code (0 on no error).                                   |
// +-------------------------------------------------------------------------+
// |CONNECT*   MailSrvr - The connection to the mail server.                 |
// |CGI*       Form     - The form processor.                                |
// |char*      Path     - The path that Mailer is located in.                |
// |char*      ServName - The name of the server that called the CGI.        |
// |INT32      Count    - A counter used in loops.                           |
// |INT32      Count2   - A counter used in loops.                           |
// |INT8       ECode    - A flag to store error values in.                   |
// |CONF       CInfo    - The structure to hold the configuration info.      |
// |bool       ChkFlag  - A flag used for various checks.                    |
// +-------------------------------------------------------------------------+
// |This function is the main body of Mailer.                                |
// +-------------------------------------------------------------------------+
int main (int argc, char** argv) {
   CONF     CInfo;

   for (INT8 Counter = 1; Counter < argc; Counter++) {
      if (argv [Counter] [0] == '-') {
         switch (argv [Counter] [1] & 0xDF) {
            case 'L': // The user requested to see the license.
                      License ();
                      exit    (0);
                      break;
            case 'V': // The user requested to display the version number.
                      printf ("Mailer v%2.2f %s (%s)\n",
                              NUMBER,
                              PLATFORM,
                              NAME                      );
                      exit   (0);
                      break;
            default:  // There was an error processing the flag.
                      printf ("ERROR:  The flag '%c' was not recognized.",
                              argv [Counter] [1]                          );
                      exit   (100);
                      break;
            }
         }
   }

   // Print the HTML header information.
   PrnHead ();

#ifdef __WIN32__
   char* Path  = strrchr (argv [0], '\\');
   *Path = 0;
   INT8 ECode = GetConf (&CInfo, argv [0]);
#elif __linux__
   INT8 ECode = GetConf (&CInfo, getcwd (NULL, 0));
#endif

   if (ECode) {
      DispError (ECode);
   }

   char* ServName = Server ();
   if (!ServName) {
      DispError (8);
   }
   
   ECode       = 1;
   INT32 Count = 0;

   // Check to see if the calling server is in the list of valid servers.
   do {
     if (!strcmpi ((CInfo.Valid [Count]), ServName)) {
        ECode = 0;
     }
   } while (++Count < CInfo.NumValid);

   if (ECode) {
      DispError (7);
   }

   CGI* Form = new CGI;
   Form->Get ();

   if (!Form->Fields) {
      DispError (Form->Status);
   }

   CInfo.ToAddr  = NULL;
   CInfo.Subject = NULL;
   CInfo.Link    = NULL;
   CInfo.Desc    = NULL;
   CInfo.Title   = NULL;

   Count = 0;

   do {
      bool ChkFlag = false;

      if (!strcmpi (Form->Fields [Count].Field, "MailTo")) {
         ChkFlag      = true;
         CInfo.ToAddr = Form->Fields [Count].Value;
      }
      else if (!strcmpi (Form->Fields [Count].Field, "Subject")) {
         ChkFlag       = true;
         CInfo.Subject = Form->Fields [Count].Value;
      }
      else if (!strcmpi (Form->Fields [Count].Field, "Link")) {
         ChkFlag      = true;
         CInfo.Link   = Form->Fields [Count].Value;
      }
      else if (!strcmpi (Form->Fields [Count].Field, "Description")) {
         ChkFlag      = true;
         CInfo.Desc   = Form->Fields [Count].Value;
      }
      else if (!strcmpi (Form->Fields [Count].Field, "Title")) {
         ChkFlag      = true;
         CInfo.Title  = Form->Fields [Count].Value;
      }

      if (ChkFlag) {
         INT32 Count2 = Count;

         do {
            Form->Fields [Count2].Field = Form->Fields [Count2 + 1].Field;
            Form->Fields [Count2].Value = Form->Fields [Count2 + 1].Value;
         } while (++Count2 < Form->NumFields - 1);

         Form->NumFields--;
         Count--;
      }
   } while (++Count < Form->NumFields);


   if (!CInfo.ToAddr) {
      DispError (9);
   }
   if (!CInfo.Subject) {
      DispError (10);
   }
   if (CInfo.Link && !CInfo.Desc) {
      DispError (11);
   }

   CONNECT* MailSrvr = new CONNECT;
   if (MailSrvr->Status) {
      DispError (MailSrvr->Status);
   }

   MailSrvr->MailInfo (*Form->Fields, CInfo, Form->NumFields, Form->Longest);
   if (MailSrvr->Status) {
     // A network error was encountered while trying to send the form to the
     // mail server.
     DispError (14);
   }

   delete MailSrvr;

   Count = 0;

   do {
      printf ( "         %s: ", Form->Fields [Count].Field);

      INT32 Count2 = 0;

      do {
        printf ("&nbsp;");
        ++Count2;
      } while (Count2 < Form->Longest - strlen (Form->Fields [Count].Field));

      printf ("<B>%s</B><BR>\n", Form->Fields [Count].Value);
   } while (++Count < Form->NumFields);

   printf  ("<BR>%9s<A Href = \"%s\">%s</A>\n", "", CInfo.Link, CInfo.Desc);

   if (!CInfo.Title) {
      PrnEnd  ("Receipt");
   }
   else {
      PrnEnd (CInfo.Title);
   }

   delete Form;

   return  (0);
}
